#!/usr/bin/env node
"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const node_path_1 = __importDefault(require("node:path"));
const node_util_1 = require("node:util");
const _1 = require(".");
const help_1 = require("./help");
const reporters_1 = __importDefault(require("./reporters"));
const errors_1 = require("./util/errors");
const { values: { help, dir, config: configFilePath, tsConfig: tsConfigFilePath, include = [], exclude = [], ignore = [], 'no-gitignore': isNoGitIgnore = false, dev: isDev = false, 'include-entry-files': isIncludeEntryFiles = false, 'no-progress': noProgress = false, reporter = 'symbols', 'reporter-options': reporterOptions = '', 'max-issues': maxIssues = '0', jsdoc: jsDoc = [], debug: isDebug = false, 'debug-level': debugLevel = '1', }, } = (0, node_util_1.parseArgs)({
    options: {
        help: { type: 'boolean' },
        config: { type: 'string', short: 'c' },
        tsConfig: { type: 'string', short: 't' },
        dir: { type: 'string' },
        include: { type: 'string', multiple: true },
        exclude: { type: 'string', multiple: true },
        ignore: { type: 'string', multiple: true },
        'no-gitignore': { type: 'boolean' },
        dev: { type: 'boolean' },
        'include-entry-files': { type: 'boolean' },
        'no-progress': { type: 'boolean' },
        'max-issues': { type: 'string' },
        reporter: { type: 'string' },
        'reporter-options': { type: 'string' },
        jsdoc: { type: 'string', multiple: true },
        debug: { type: 'boolean' },
        'debug-level': { type: 'string' },
    },
});
if (help) {
    (0, help_1.printHelp)();
    process.exit(0);
}
const cwd = process.cwd();
const workingDir = dir ? node_path_1.default.resolve(dir) : cwd;
const isShowProgress = noProgress === false ? process.stdout.isTTY && typeof process.stdout.cursorTo === 'function' : !noProgress;
const printReport = reporter in reporters_1.default ? reporters_1.default[reporter] : require(node_path_1.default.join(workingDir, reporter));
const run = async () => {
    try {
        const { report, issues, counters } = await (0, _1.main)({
            cwd,
            workingDir,
            configFilePath,
            tsConfigFilePath,
            include,
            exclude,
            ignore,
            gitignore: !isNoGitIgnore,
            isIncludeEntryFiles,
            isDev,
            isShowProgress,
            jsDoc,
            debug: {
                isEnabled: isDebug,
                level: isDebug ? Number(debugLevel) : 0,
            },
        });
        printReport({ report, issues, cwd, workingDir, isDev, options: reporterOptions });
        const reportGroup = report.files ? 'files' : Object.keys(report).find(key => report[key]);
        const counterGroup = reportGroup === 'unlisted' ? 'unresolved' : reportGroup;
        if (counterGroup) {
            const count = counters[counterGroup];
            if (count > Number(maxIssues))
                process.exit(count);
        }
    }
    catch (error) {
        if (error instanceof errors_1.ConfigurationError) {
            console.error(error.message + '\n');
            (0, help_1.printHelp)();
            process.exit(1);
        }
        throw error;
    }
};
run();
