"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const node_path_1 = __importDefault(require("node:path"));
const ownership_1 = require("@snyk/github-codeowners/dist/lib/ownership");
const logIssueLine = (owner, cwd, filePath, symbols) => {
    console.log(`${owner} ${node_path_1.default.relative(cwd, filePath)}${symbols ? `: ${symbols.join(', ')}` : ''}`);
};
const logIssueGroupResult = (issues, cwd, title) => {
    title && console.log(`--- ${title} (${issues.length})`);
    if (issues.length) {
        issues
            .sort((a, b) => (a.owner < b.owner ? -1 : 1))
            .forEach(issue => console.log(issue.owner, issue.symbol.startsWith('/') ? node_path_1.default.relative(cwd, issue.symbol) : issue.symbol));
    }
    else {
        console.log('Not found');
    }
};
const logIssueGroupResults = (issues, cwd, title) => {
    title && console.log(`--- ${title} (${issues.length})`);
    if (issues.length) {
        const sortedByFilePath = issues.sort((a, b) => (a.owner < b.owner ? -1 : 1));
        sortedByFilePath.forEach(({ filePath, symbols, owner }) => logIssueLine(owner, cwd, filePath, symbols));
    }
    else {
        console.log('Not found');
    }
};
exports.default = ({ report, issues, cwd, isDev, options }) => {
    let opts = {};
    try {
        opts = options ? JSON.parse(options) : opts;
    }
    catch (error) {
        console.error(error);
    }
    const codeownersFilePath = node_path_1.default.resolve(opts.path ?? '.github/CODEOWNERS');
    const codeownersEngine = ownership_1.OwnershipEngine.FromCodeownersFile(codeownersFilePath);
    const reportMultipleGroups = Object.values(report).filter(Boolean).length > 1;
    const [dependenciesOwner = '[no-owner]'] = codeownersEngine.calcFileOwnership('package.json');
    const fallbackOwner = dependenciesOwner;
    const calcFileOwnership = (filePath) => codeownersEngine.calcFileOwnership(node_path_1.default.relative(cwd, filePath))[0] ?? fallbackOwner;
    const toIssueWithOwner = (issues) => {
        const items = Object.values(issues);
        return { ...items[0], symbols: items.map(i => i.symbol), owner: calcFileOwnership(items[0].filePath) };
    };
    if (report.files) {
        const unreferencedFilesByOwner = Array.from(issues.files).map(filePath => ({
            symbol: filePath,
            owner: calcFileOwnership(filePath),
        }));
        logIssueGroupResult(unreferencedFilesByOwner, cwd, reportMultipleGroups && 'UNUSED FILES');
    }
    if (report.dependencies) {
        const unreferencedDependencies = Array.from(issues.dependencies).map(dependency => ({
            symbol: dependency,
            owner: dependenciesOwner,
        }));
        logIssueGroupResult(unreferencedDependencies, cwd, reportMultipleGroups && 'UNUSED DEPENDENCIES');
    }
    if (report.dependencies && isDev) {
        const unreferencedDevDependencies = Array.from(issues.devDependencies).map(dependency => ({
            symbol: dependency,
            owner: dependenciesOwner,
        }));
        logIssueGroupResult(unreferencedDevDependencies, cwd, 'UNUSED DEV DEPENDENCIES');
    }
    if (report.unlisted) {
        const unreferencedDependencies = Object.values(issues.unresolved).map(toIssueWithOwner);
        logIssueGroupResults(unreferencedDependencies, cwd, reportMultipleGroups && 'UNLISTED DEPENDENCIES');
    }
    if (report.exports) {
        const unreferencedExports = Object.values(issues.exports).map(toIssueWithOwner);
        logIssueGroupResults(unreferencedExports, cwd, reportMultipleGroups && 'UNUSED EXPORTS');
    }
    if (report.nsExports) {
        const unreferencedNsExports = Object.values(issues.nsExports).map(toIssueWithOwner);
        logIssueGroupResults(unreferencedNsExports, cwd, reportMultipleGroups && 'UNUSED EXPORTS IN NAMESPACE');
    }
    if (report.types) {
        const unreferencedTypes = Object.values(issues.types).map(toIssueWithOwner);
        logIssueGroupResults(unreferencedTypes, cwd, reportMultipleGroups && 'UNUSED TYPES');
    }
    if (report.nsTypes) {
        const unreferencedNsTypes = Object.values(issues.nsTypes).map(toIssueWithOwner);
        logIssueGroupResults(unreferencedNsTypes, cwd, reportMultipleGroups && 'UNUSED TYPES IN NAMESPACE');
    }
    if (report.duplicates) {
        const unreferencedDuplicates = Object.values(issues.duplicates)
            .map(issues => Object.values(issues))
            .flat()
            .map(issue => ({ ...issue, owner: calcFileOwnership(issue.filePath) }));
        logIssueGroupResults(unreferencedDuplicates, cwd, reportMultipleGroups && 'DUPLICATE EXPORTS');
    }
};
