"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveIncludedIssueGroups = exports.resolveConfig = void 0;
const micromatch_1 = __importDefault(require("micromatch"));
const resolveConfig = (importedConfiguration, options) => {
    if (!importedConfiguration)
        return;
    let resolvedConfig = importedConfiguration;
    const { workingDir, isDev } = options ?? {};
    const configKeys = Object.keys(importedConfiguration);
    if (workingDir && !('projectFiles' in importedConfiguration)) {
        const importedConfigKey = configKeys.find(pattern => micromatch_1.default.isMatch(workingDir.replace(/\/$/, ''), pattern));
        if (importedConfigKey) {
            resolvedConfig = importedConfiguration[importedConfigKey];
        }
    }
    if (isDev && typeof resolvedConfig.dev === 'object' && 'projectFiles' in resolvedConfig.dev) {
        resolvedConfig = resolvedConfig.dev;
    }
    if (!resolvedConfig.entryFiles || !resolvedConfig.projectFiles) {
        console.info(`Add these properties at root level, or use --dir and match one of: ${configKeys.join(', ')}\n`);
        return;
    }
    return resolvedConfig;
};
exports.resolveConfig = resolveConfig;
const resolveIncludedIssueGroups = (includeArg, excludeArg, resolvedConfig) => {
    const groups = [
        'files',
        'dependencies',
        'unlisted',
        'exports',
        'types',
        'nsExports',
        'nsTypes',
        'duplicates',
    ];
    const include = [includeArg, resolvedConfig?.include ?? []]
        .flat()
        .map(value => value.split(','))
        .flat();
    const exclude = [excludeArg, resolvedConfig?.exclude ?? []]
        .flat()
        .map(value => value.split(','))
        .flat();
    const includes = (include.length > 0 ? include : groups).filter((group) => !exclude.includes(group));
    return groups.reduce((r, group) => ((r[group] = includes.includes(group)), r), {});
};
exports.resolveIncludedIssueGroups = resolveIncludedIssueGroups;
