"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDependencyAnalyzer = void 0;
const ts_morph_1 = require("ts-morph");
const is_builtin_module_1 = __importDefault(require("is-builtin-module"));
const micromatch_1 = __importDefault(require("micromatch"));
const compact = (collection) => Array.from(new Set(collection)).filter((value) => Boolean(value));
const getDependencyAnalyzer = (configuration) => {
    const { dependencies, devDependencies, peerDependencies, optionalDependencies, tsConfigPaths } = configuration;
    const productionDependencies = [...dependencies, ...peerDependencies, ...optionalDependencies];
    const referencedDependencies = new Set();
    const getUnresolvedDependencies = (sourceFile) => {
        const unresolvedDependencies = new Set();
        const importLiterals = sourceFile.getImportStringLiterals();
        const requires = sourceFile
            .getDescendantsOfKind(ts_morph_1.ts.SyntaxKind.CallExpression)
            .filter(callExpression => callExpression.getExpression().getText() === 'require')
            .map(expression => expression.getFirstDescendantByKind(ts_morph_1.ts.SyntaxKind.StringLiteral));
        const literals = compact([importLiterals, requires].flat());
        literals.forEach(importLiteral => {
            const moduleSpecifier = importLiteral.getLiteralText();
            if (moduleSpecifier.startsWith('.'))
                return;
            if ((0, is_builtin_module_1.default)(moduleSpecifier))
                return;
            if (tsConfigPaths.length > 0 && micromatch_1.default.isMatch(moduleSpecifier, tsConfigPaths))
                return;
            const parts = moduleSpecifier.split('/').slice(0, 2);
            const packageName = moduleSpecifier.startsWith('@') ? parts.join('/') : parts[0];
            if (!productionDependencies.includes(packageName) && !devDependencies.includes(packageName)) {
                unresolvedDependencies.add({ filePath: sourceFile.getFilePath(), symbol: moduleSpecifier });
            }
            if (productionDependencies.includes(packageName) || devDependencies.includes(packageName)) {
                referencedDependencies.add(packageName);
            }
        });
        return unresolvedDependencies;
    };
    const getUnusedDependencies = () => productionDependencies.filter(dependency => !referencedDependencies.has(dependency));
    const getUnusedDevDependencies = () => devDependencies.filter(dependency => !referencedDependencies.has(dependency));
    return { getUnresolvedDependencies, getUnusedDependencies, getUnusedDevDependencies };
};
exports.getDependencyAnalyzer = getDependencyAnalyzer;
